'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.commands = undefined;

var _utils = require('./utils');

var _app = require('./app');

var _package = require('./package');

/**
 * Created by tdzl2003 on 4/2/16.
 */

const {
  get,
  post,
  put,
  uploadFile
} = require('./api');


async function showVersion(appId, offset) {
  const { data, count } = await get(`/app/${appId}/version/list`);
  console.log(`Offset ${offset}`);
  for (const version of data) {
    let packageInfo = version.packages.slice(0, 3).map(v => v.name).join(', ');
    const count = version.packages.length;
    if (count > 3) {
      packageInfo += `...and ${count - 3} more`;
    }
    if (count === 0) {
      packageInfo = `(no package)`;
    } else {
      packageInfo = `[${packageInfo}]`;
    }
    console.log(`${version.id}) ${version.hash.slice(0, 8)} ${version.name} ${packageInfo}`);
  }
  return data;
}

async function listVersions(appId) {
  let offset = 0;
  while (true) {
    await showVersion(appId, offset);
    const cmd = await (0, _utils.question)('page Up/page Down/Begin/Quit(U/D/B/Q)');
    switch (cmd.toLowerCase()) {
      case 'u':
        offset = Math.max(0, offset - 10);break;
      case 'd':
        offset += 10;break;
      case 'b':
        offset = 0;break;
      case 'q':
        return;
    }
  }
}

async function chooseVersion(appId) {
  let offset = 0;
  while (true) {
    const data = await showVersion(appId, offset);
    const cmd = await (0, _utils.question)('Enter versionId or page Up/page Down/Begin(U/D/B)');
    switch (cmd.toLowerCase()) {
      case 'U':
        offset = Math.max(0, offset - 10);break;
      case 'D':
        offset += 10;break;
      case 'B':
        offset = 0;break;
      default:
        {
          const v = data.find(v => v.id === (cmd | 0));
          if (v) {
            return v;
          }
        }
    }
  }
}

const commands = exports.commands = {
  publish: async function ({ args, options }) {
    const fn = args[0];
    const { name, description, metaInfo } = options;

    if (!fn || !fn.endsWith('.ppk')) {
      throw new Error('Usage: pushy publish <ppkFile> --platform ios|android');
    }

    const platform = (0, _app.checkPlatform)(options.platform || (await (0, _utils.question)('Platform(ios/android):')));
    const { appId } = await (0, _app.getSelectedApp)(platform);

    const { hash } = await uploadFile(fn);

    const { id } = await post(`/app/${appId}/version/create`, {
      name: name || (await (0, _utils.question)('Enter version name:')) || '(未命名)',
      hash,
      description: description || (await (0, _utils.question)('Enter description:')),
      metaInfo: metaInfo || (await (0, _utils.question)('Enter meta info:'))
    });
    // TODO local diff
    (0, _utils.saveToLocal)(fn, `${appId}/ppk/${id}.ppk`);
    console.log(`Version published: ${id}`);

    const v = await (0, _utils.question)('Would you like to bind packages to this version?(Y/N)');
    if (v.toLowerCase() === 'y') {
      await this.update({ args: [], options: { versionId: id, platform } });
    }
  },
  versions: async function ({ options }) {
    const platform = (0, _app.checkPlatform)(options.platform || (await (0, _utils.question)('Platform(ios/android):')));
    const { appId } = await (0, _app.getSelectedApp)(platform);
    await listVersions(appId);
  },
  update: async function ({ args, options }) {
    const platform = (0, _app.checkPlatform)(options.platform || (await (0, _utils.question)('Platform(ios/android):')));
    const { appId } = await (0, _app.getSelectedApp)(platform);
    const versionId = options.versionId || (await chooseVersion(appId)).id;
    const pkgId = options.packageId || (await (0, _package.choosePackage)(appId)).id;
    await put(`/app/${appId}/package/${pkgId}`, {
      versionId
    });
    console.log('Ok.');
  }
};